#
# Cookbook Name:: Venafi TPP
# Recipe:: default
#
# Copyright 2015, Venafi
#
# All rights reserved - Do Not Redistribute
#

require "base64"
#include_recipe 'webhooks::tpp_authenticate'

#############################################################################################################################



webhooks_request "certificates_retrieve" do
  uri lazy {"#{node["tpp"]["server"]}/vedsdk/certificates/retrieve?apikey=#{node["tpp"]["apikey"]}"}
  ssl_validation false
  use_ssl true
  follow_redirect true
  post_json true
  post_data lazy {(
            { 'CertificateDN' => node["tpp"]["certificate_dn"],
              'Format' => node["tpp"]["key_format"],
              'Password' => node["tpp"]["key_password"],
              'IncludePrivateKey' => node["tpp"]["include_private_key"],
              'FriendlyName' => node["tpp"]["certificate_dn"],
              'IncludeChain' => true }
            )}
  header_data (
                  { 'content-type' => 'application/json', 'auth' => "#{node["tpp"]["apikey"]} }" }
              )
  expected_response_codes [ 200, 202, 201, 400, 401 ]
  action :nothing
end


ruby_block "Retrieve Certificate" do
  block do

    counter=1
    while counter <= node["tpp"]["max_retry_times"]

      resources(:webhooks_request => "certificates_retrieve").run_action(:post)
      Chef::Log.info "Retrieve certificate attempt #{counter} of #{node["tpp"]["max_retry_times"]}"

      Chef::Log.info "==============================================================================="
      Chef::Log.info "TPP response message: " + node["webhooks"]["response_message"]
      Chef::Log.info "TPP response code: " + node["webhooks"]["response_code"]
      Chef::Log.info "==============================================================================="

        jsonData = JSON.parse(node["webhooks"]["post_response"])

      if jsonData['CertificateData']
        decode_base64_content = Base64.decode64(jsonData['CertificateData'])
        node.set["tpp"]["certificate"] = decode_base64_content
        #Chef::Log.info node["tpp"]["certificate"]
        Chef::Log.info "Suggested filename: #{jsonData['Filename']}"
        Chef::Log.info "File format: #{jsonData['Format']}"
        Chef::Log.info "Writing to file: #{node["tpp"]["certificate_file_path"]}#{jsonData['Filename']} "
        File.open(node["tpp"]["certificate_file_path"] + jsonData['Filename'], 'wb') { |file| file.write(node["tpp"]["certificate"]) }
        break


      else if jsonData['Stage']
          Chef::Log.info "Certificate not ready, going to wait for #{node["tpp"]["sleep_seconds"]} seconds because TPP is at Processing Stage #{jsonData['Stage']}"
          Chef::Log.info "Current TPP Processing Status #{jsonData['Status']}"
          sleep node["tpp"]["sleep_seconds"]
           end

      end
      counter+=1

       end
  end
  action :run
end
